
//
// Licensed to the Apache Software Foundation (ASF) under one
// or more contributor license agreements.  See the NOTICE file
// distributed with this work for additional information
// regarding copyright ownership.  The ASF licenses this file
// to you under the Apache License, Version 2.0 (the
// "License"); you may not use this file except in compliance
// with the License.  You may obtain a copy of the License at
// 
//   http://www.apache.org/licenses/LICENSE-2.0
// 
// Unless required by applicable law or agreed to in writing,
// software distributed under the License is distributed on an
// "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
// KIND, either express or implied.  See the License for the
// specific language governing permissions and limitations
// under the License.
//

// This source file was created by a code generator.
// Please do not edit.

#include "qpid/log/Statement.h"
#include "qpid/framing/FieldTable.h"
#include "qpid/management/Manageable.h" 
#include "qpid/agent/ManagementAgent.h"
#include "Host.h"


using namespace qmf::com::redhat::matahari;
using namespace qpid::framing;
using           qpid::management::ManagementAgent;
using           qpid::management::Manageable;
using           qpid::management::ManagementObject;
using           qpid::management::Args;
using           std::string;

string  Host::packageName  = string ("com.redhat.matahari");
string  Host::className    = string ("host");
uint8_t Host::md5Sum[16]   =
    {0x1a,0xeb,0x2,0x4,0xc3,0xf0,0x4,0x40,0x82,0x79,0xf3,0x5c,0xdf,0xc8,0xab,0x55};

Host::Host (ManagementAgent*, Manageable* _core) :
    ManagementObject(_core)
{
    
    uuid = "";
    hostname = "";
    memory = 0;
    hypervisor = "";
    arch = "";
    beeping = 0;



}

Host::~Host ()
{

}

namespace {
    const string NAME("name");
    const string TYPE("type");
    const string ACCESS("access");
    const string IS_INDEX("index");
    const string IS_OPTIONAL("optional");
    const string UNIT("unit");
    const string MIN("min");
    const string MAX("max");
    const string MAXLEN("maxlen");
    const string DESC("desc");
    const string ARGCOUNT("argCount");
    const string ARGS("args");
    const string DIR("dir");
    const string DEFAULT("default");
}

void Host::registerSelf(ManagementAgent* agent)
{
    agent->registerClass(packageName, className, md5Sum, writeSchema);
}

void Host::writeSchema (Buffer& buf)
{
    FieldTable ft;

    // Schema class header:
    buf.putOctet       (CLASS_KIND_TABLE);
    buf.putShortString (packageName); // Package Name
    buf.putShortString (className);   // Class Name
    buf.putBin128      (md5Sum);      // Schema Hash
    buf.putOctet       (0);           // No Superclass
    buf.putShort       (6); // Config Element Count
    buf.putShort       (0); // Inst Element Count
    buf.putShort       (2); // Method Count

    // Properties
    ft.clear();
    ft.setString (NAME, "uuid");
    ft.setInt    (TYPE, TYPE_SSTR);
    ft.setInt    (ACCESS, ACCESS_RO);
    ft.setInt    (IS_INDEX, 1);
    ft.setInt    (IS_OPTIONAL, 0);
    ft.setString (DESC,   "Host UUID");
    buf.put (ft);

    ft.clear();
    ft.setString (NAME, "hostname");
    ft.setInt    (TYPE, TYPE_SSTR);
    ft.setInt    (ACCESS, ACCESS_RO);
    ft.setInt    (IS_INDEX, 0);
    ft.setInt    (IS_OPTIONAL, 0);
    ft.setString (DESC,   "Hostname");
    buf.put (ft);

    ft.clear();
    ft.setString (NAME, "memory");
    ft.setInt    (TYPE, TYPE_S32);
    ft.setInt    (ACCESS, ACCESS_RO);
    ft.setInt    (IS_INDEX, 0);
    ft.setInt    (IS_OPTIONAL, 0);
    ft.setString (UNIT,   "kb");
    ft.setString (DESC,   "Amount of primary memory for host (kb)");
    buf.put (ft);

    ft.clear();
    ft.setString (NAME, "hypervisor");
    ft.setInt    (TYPE, TYPE_SSTR);
    ft.setInt    (ACCESS, ACCESS_RO);
    ft.setInt    (IS_INDEX, 0);
    ft.setInt    (IS_OPTIONAL, 0);
    ft.setString (DESC,   "Hypervisor used by host");
    buf.put (ft);

    ft.clear();
    ft.setString (NAME, "arch");
    ft.setInt    (TYPE, TYPE_SSTR);
    ft.setInt    (ACCESS, ACCESS_RO);
    ft.setInt    (IS_INDEX, 0);
    ft.setInt    (IS_OPTIONAL, 0);
    ft.setString (DESC,   "Architecture of host");
    buf.put (ft);

    ft.clear();
    ft.setString (NAME, "beeping");
    ft.setInt    (TYPE, TYPE_BOOL);
    ft.setInt    (ACCESS, ACCESS_RW);
    ft.setInt    (IS_INDEX, 0);
    ft.setInt    (IS_OPTIONAL, 0);
    ft.setString (DESC,   "speaker beep loop active? (used to identify host)");
    buf.put (ft);


    // Statistics

    // Methods
    ft.clear();
    ft.setString (NAME,     "shutdown");
    ft.setInt    (ARGCOUNT, 0);
    ft.setString (DESC,     "Shutdown node");
    buf.put (ft);

    ft.clear();
    ft.setString (NAME,     "reboot");
    ft.setInt    (ARGCOUNT, 0);
    ft.setString (DESC,     "Reboot node");
    buf.put (ft);


}



void Host::writeProperties (Buffer& buf)
{
    ::qpid::sys::Mutex::ScopedLock mutex(accessLock);
    configChanged = false;

    writeTimestamps (buf);

    buf.putShortString(uuid);
    buf.putShortString(hostname);
    buf.putInt32(memory);
    buf.putShortString(hypervisor);
    buf.putShortString(arch);
    buf.putOctet(beeping?1:0);

}

void Host::writeStatistics (Buffer& buf, bool skipHeaders)
{
    ::qpid::sys::Mutex::ScopedLock mutex(accessLock);
    instChanged = false;



    if (!skipHeaders)
        writeTimestamps (buf);


    // Maintenance of hi-lo statistics


}

void Host::doMethod (string& methodName, Buffer&, Buffer& outBuf)
{
    Manageable::status_t status = Manageable::STATUS_UNKNOWN_METHOD;
    std::string          text;


    if (methodName == "shutdown") {
        ::qpid::management::ArgsNone ioArgs;
        status = coreObject->ManagementMethod (METHOD_SHUTDOWN, ioArgs, text);
        outBuf.putLong        (status);
        outBuf.putMediumString(::qpid::management::Manageable::StatusText (status, text));
        return;
    }

    if (methodName == "reboot") {
        ::qpid::management::ArgsNone ioArgs;
        status = coreObject->ManagementMethod (METHOD_REBOOT, ioArgs, text);
        outBuf.putLong        (status);
        outBuf.putMediumString(::qpid::management::Manageable::StatusText (status, text));
        return;
    }

    outBuf.putLong(status);
    outBuf.putShortString(Manageable::StatusText(status, text));
}
